#! /usr/bin/python
# -*- coding: utf8 -*-

# Time filtering of gp2x's stick events.
# by Radek Kowalski 2006
# This is version two with added regulation for dynamic gating.
# (the former "pseudointelligence")
# Public Domain and use at your own risk.

# GP2X joystick button mappings
BUTTON_UP            = (0)
BUTTON_DOWN          = (4)
BUTTON_LEFT          = (2)
BUTTON_RIGHT         = (6)
BUTTON_UPLEFT        = (1)
BUTTON_UPRIGHT       = (7)
BUTTON_DOWNLEFT      = (3)
BUTTON_DOWNRIGHT     = (5)
BUTTON_CLICK         = (18)
BUTTON_A             = (12)
BUTTON_B             = (13)
BUTTON_X             = (14)
BUTTON_Y             = (15)
BUTTON_L             = (10)
BUTTON_R             = (11)
BUTTON_START         = (8) 
BUTTON_SELECT        = (9)
BUTTON_VOLUP         = (16)
BUTTON_VOLDOWN       = (17)

from pygame import *

display.init()
scr = display.set_mode((320, 240))

background = image.load("timegate.png")
scr.blit(background, (0, 0))

# initialisation
joystick.init()
num_joysticks = joystick.get_count()
if num_joysticks > 0:
    stick = joystick.Joystick(0)
    stick.init() # now we will receive events for the joystick

def SetStick(StickButtons, ev):
    if ev.button == BUTTON_UP:   Switch = 0
    elif ev.button == BUTTON_DOWN: Switch = 4
    elif ev.button == BUTTON_LEFT:  Switch = 6
    elif ev.button == BUTTON_RIGHT: Switch = 2
    elif ev.button == BUTTON_UPLEFT: Switch = 7
    elif ev.button == BUTTON_UPRIGHT: Switch = 1
    elif ev.button == BUTTON_DOWNLEFT: Switch = 5
    elif ev.button == BUTTON_DOWNRIGHT:  Switch = 3
    elif ev.button == BUTTON_CLICK: Switch = 8
    else:
        return 0
    if ev.type == JOYBUTTONDOWN:
        StickButtons[Switch] = 1
    elif ev.type == JOYBUTTONUP:
        StickButtons[Switch] = 0
    else:
        StickButtons = [0, 1, 0, 1, 0, 1, 0, 1, 0]
        return 0
    
def SetStickWithKeyboard(StickButtons, ev):
    """This procedure is very usefull to debugging code on a PC."""
    if ev.key == 119:   Switch = 0
    elif ev.key == 120: Switch = 4
    elif ev.key == 97:  Switch = 6
    elif ev.key == 100: Switch = 2
    elif ev.key == 113: Switch = 7
    elif ev.key == 101: Switch = 1
    elif ev.key == 122: Switch = 5
    elif ev.key == 99:  Switch = 3
    elif ev.key == 115: Switch = 8
    else:
        return 0
    if ev.type == KEYDOWN:
        StickButtons[Switch] = 1
    elif ev.type == KEYUP:
        StickButtons[Switch] = 0
    else:
        StickButtons = [0, 1, 0, 1, 0, 1, 0, 1, 0]
        return 0

def StickFilter(StickButtons, StickDelay, OrdFilter = 1, DiaFilter = 1,
                                            PseudoIntel = 0, PsIntDelay = [0]):
    NotIddle = sum(StickButtons[:-1])
    if NotIddle == 0 and PsIntDelay[0] == 0 and PseudoIntel > 0:
        for x in [0, 1, 2, 3, 4, 5, 6, 7]:
            StickDelay[x] = 0
    else:
        for x in [0, 1, 2, 3, 4, 5, 6, 7]:
            if StickButtons[x] == 0:
                if x % 2:       # for diagonals
                    StickDelay[x] = DiaFilter
                else:           # for ordinals
                    StickDelay[x] = OrdFilter
            else:
                StickDelay[x] -= (StickDelay[x] > 0) and 1
    if NotIddle:
        PsIntDelay[0] = 8 - PseudoIntel
    else:
        PsIntDelay[0] -= (PsIntDelay[0] > 0) and 1

def StickOutput(StickButtons, StickDelay, StickOut):
    for x in [0, 1, 2, 3, 4, 5, 6, 7]:
        if StickButtons[x] == 1 and StickDelay[x] == 0:
            StickOut[x] = 1
        else:
            StickOut[x] = 0
    StickOut[8] = StickButtons[8]

def StickPreFilter(Stick, Out):
    for x in [0, 1, 2, 3, 4, 5, 6, 7, 8]:
        Out[x] = Stick[x]
    if Stick[0] and Stick[1]:
        Out[1] = 0
    elif Stick[1] and Stick[2]:
        Out[1] = 0
    elif Stick[2] and Stick[3]:
        Out[3] = 0
    elif Stick[3] and Stick[4]:
        Out[3] = 0
    elif Stick[4] and Stick[5]:
        Out[5] = 0
    elif Stick[5] and Stick[6]:
        Out[5] = 0
    elif Stick[6] and Stick[7]:
        Out[7] = 0
    elif Stick[7] and Stick[0]:
        Out[7] = 0

def DrawStickPositions(scr, StickButtons, (posX, posY) = (10, 10),
                                rectangleSize = 30, rectangleSpacing = 50):
    StickM = ((7, 0, 1), (6, 8, 2), (5, 4, 3))
    for y in [0, 1, 2]:
        for x in [0, 1, 2]:
            if StickButtons[StickM[y][x]] > 0:
                color = (255, 0, 0)
            else:
                color = (255, 255, 255)
            draw.rect(scr, color, (x * rectangleSpacing + posX,
                                             y * rectangleSpacing + posY,
                                             rectangleSize, rectangleSize))

def DrawVerticalBar(scr, Val, MaxVal, (posX, posY), recSize, recSpacing):
    for y in range(MaxVal):
        if (MaxVal - y - 1) < Val:
            draw.rect(scr, (255, 0, 0), (posX, posY + recSpacing * y, recSize, recSize), 0)
        else:
            draw.rect(scr, (255, 255, 255), (posX, posY + recSpacing * y, recSize, recSize), 0)

StickButtons = [0, 0, 0, 0, 0, 0, 0, 0, 0]
StickDelay = [0, 0, 0, 0, 0, 0, 0, 0, 0]
SOut = [0, 0, 0, 0, 0, 0, 0, 0, 0]
SOut2 = [0, 0, 0, 0, 0, 0, 0, 0, 0]

OrdinalFiltering = 1
DiagonalFiltering = 1
PseudoIntel = 0
PsIntDelay = [0]

while True:
    try:
        e = event.poll()
        if e.type != NOEVENT:
            if e.type == JOYBUTTONDOWN:
                if e.button == BUTTON_START or e.button == BUTTON_SELECT:
                    break
                elif e.button == BUTTON_B:
                    OrdinalFiltering += 1
                elif e.button == BUTTON_A:
                    OrdinalFiltering -= 1
                elif e.button == BUTTON_Y:
                    DiagonalFiltering += 1
                elif e.button == BUTTON_X:
                    DiagonalFiltering -= 1
                elif e.button == BUTTON_VOLDOWN:
                    PseudoIntel -= 1
                elif e.button == BUTTON_VOLUP:
                    PseudoIntel += 1
                if OrdinalFiltering > 9: OrdinalFiltering = 9
                if OrdinalFiltering < 1: OrdinalFiltering = 1
                if DiagonalFiltering > 9: DiagonalFiltering = 9
                if DiagonalFiltering < 1: DiagonalFiltering = 1
                if PseudoIntel > 8: PseudoIntel = 8
                if PseudoIntel < 0: PseudoIntel = 0
            SetStick(StickButtons, e)
        StickPreFilter(StickButtons, SOut)
        
        StickFilter(SOut, StickDelay, OrdinalFiltering, DiagonalFiltering,
                                                    PseudoIntel, PsIntDelay)
        StickOutput(SOut, StickDelay, SOut2)
        DrawStickPositions(scr, StickButtons, (8,8), 20, 25)
        DrawStickPositions(scr, SOut, (127, 8), 20, 25)
        DrawStickPositions(scr, SOut2, (243, 8), 20, 25)

        DrawVerticalBar(scr, OrdinalFiltering - 1, 8, (37, 98), 12, 16)
        DrawVerticalBar(scr, DiagonalFiltering - 1, 8, (272, 98), 12, 16)
        DrawVerticalBar(scr, PseudoIntel, 8, (156, 98), 12, 16)
        
        display.flip()
        #time.wait(100)  # not needed on gp2x but usefull when running on a PC :)
    except:
        pass
